// var async = require('asyncawait/async')
// var await = require('asyncawait/await')
var Horseman = require('node-horseman');
var horseman = new Horseman();
var axios = require('axios');
var chalk = require('chalk');
var fs = require('fs');
var download = require('download');
var rc = require('vorpal-rc');
var path = require('path');
var os = require('os');
axios.defaults.baseURL = 'https://slate.sheridancollege.ca/d2l/api/';

var config = null;

const vorpal = require('vorpal')();

const repl = require('vorpal-repl');
var Table = require('cli-table');

var globalCourseListing = null;

var getClassList = async function (courseId) {
    var res = await axios.get('/le/1.10/' + courseId + '/classlist/');
    return res.data;
};

var getContentToc = async function (courseId) {
    var res = await axios.get('/le/1.10/' + courseId + '/content/toc');
    console.log(res);
};

var getCourseGradeDetail = async function (courseId, gradeId) {
    var res = await axios.get('/le/1.10/' + courseId + '/grades/' + gradeId + '/values/myGradeValue').catch(function (err) {
        return { data: { "DisplayedGrade": "None" } };
    });
    return res.data;
};

var getCourseGrade = async function (courseId) {
    var res = await axios.get('/le/1.10/' + courseId + '/grades/');
    return res.data;
};

var downloadDropboxFiles = async function (courseId, dropboxId) {
    var res = await axios.get('/le/1.10/' + courseId + '/dropbox/folders/' + dropboxId);

    res.data.Attachments.forEach(function (i) {
        download('https://slate.sheridancollege.ca/d2l/api/le/1.10/' + courseId + '/dropbox/folders/' + dropboxId + '/attachments/' + i.FileId, {
            headers: {
                cookie: axios.defaults.headers.common['Cookie']
            }
        }).pipe(fs.createWriteStream(i.FileName));
        vorpal.log("Finish Download: " + i.FileName);
    });
};

var getCourseDropbox = async function (courseId) {
    var res = await axios.get('/le/1.10/' + courseId + '/dropbox/folders/');
    return res.data;
};

var courseListing = async function () {
    if (globalCourseListing == null) {
        var res = await axios.get('/lp/1.10/enrollments/myenrollments/');

        globalCourseListing = res.data.Items.filter(function (i) {
            return i.OrgUnit.Type.Code == 'Course Offering';
        });
        return globalCourseListing;
    } else {
        return globalCourseListing;
    }
};

var findCourse = async function (course) {
    courses = await courseListing();
    return courses.find(function (i) {
        return i.OrgUnit.Name.split(' ')[0] == course;
    });
};

var courseCompletion = async function () {
    courses = await courseListing();
    return courses.map(function (i) {
        return i.OrgUnit.Name.split(' ')[0];
    });
};

var grabCookie = function () {
    return horseman.userAgent('Mozilla/5.0 (Windows NT 6.1; WOW64; rv:27.0) Gecko/20100101 Firefox/27.0').open("https://slate.sheridancollege.ca").type('input[name="j_username"]', config.username).type('input[name="j_password"]', config.password).click('[name="_eventId_proceed"]').waitForNextPage().waitForSelector('.d2l-minibar-home-text').cookies().then(function (cookies) {
        vorpal.log(chalk.green('Successfully Login'));
        axios.defaults.headers.common['Cookie'] = cookies[0].name + '=' + cookies[0].value + ';' + cookies[1].name + '=' + cookies[1].value;

        var data = {
            username: config.username,
            password: config.password,
            cookie: axios.defaults.headers.common['Cookie']
        };

        fs.writeFileSync(path.join(os.homedir(), '.slaterc'), JSON.stringify(data));

        config = data;
    }).close();
};

var checkCookieStatus = async function () {
    if (config.cookie == "none") {
        return false;
    } else {
        var res = await axios.get('lp/1.10/users/whoami', {
            headers: {
                cookie: config.cookie
            }
        }).then(function (res) {
            axios.defaults.headers.common['Cookie'] = config.cookie;
            vorpal.log(chalk.green('Successfully Login'));
            return true;
        }).catch(function (err) {
            return false;
        });
        return res;
    }
};

var initVorpalSlate = async function (username, password) {

    if (config == null) {
        var data = {
            username: username,
            password: password,
            cookie: "none"
        };

        fs.writeFileSync(path.join(os.homedir(), '.slaterc'), JSON.stringify(data));

        config = data;
    }

    if (!checkCookieStatus()) {
        await grabCookie();
    }

    vorpal.command('course <course>', 'list course of sheridan').option('--list-class', 'list the student of the course').option('--grades', 'list grades of the course').option('--dropbox', 'list dropbox/assignments').option('--download <dropboxId>', 'use with dropbox files').autocomplete((await courseCompletion())).action(async function (args, callback) {
        if (args.options['list-class']) {
            course = await findCourse(args.course);
            classlist = await getClassList(course.OrgUnit.Id);
            var table = new Table({
                head: ['Name'],
                chars: { 'top': '-', 'top-mid': '+', 'top-left': '+', 'top-right': '+',
                    'bottom': '-', 'bottom-mid': '+', 'bottom-left': '+', 'bottom-right': '+',
                    'left': '|', 'left-mid': '+', 'mid': '-', 'mid-mid': '+',
                    'right': '|', 'right-mid': '+', 'middle': '|' }
            });

            classlist.map(function (i) {
                table.push([i.FirstName + " " + i.LastName]);
            });
            vorpal.log(table.toString());
        } else if (args.options['dropbox']) {

            if (args.options['download']) {
                course = await findCourse(args.course);
                await downloadDropboxFiles(course.OrgUnit.Id, args.options.download);
            } else {
                course = await findCourse(args.course);
                dropbox = await getCourseDropbox(course.OrgUnit.Id);

                var table = new Table({
                    head: ['ID', 'Name', 'Due Date', 'Dropbox Close Date', 'Total Submitted'],
                    chars: { 'top': '-', 'top-mid': '+', 'top-left': '+', 'top-right': '+',
                        'bottom': '-', 'bottom-mid': '+', 'bottom-left': '+', 'bottom-right': '+',
                        'left': '|', 'left-mid': '+', 'mid': '-', 'mid-mid': '+',
                        'right': '|', 'right-mid': '+', 'middle': '|' }
                });

                dropbox.map(function (i) {

                    var dueDate = i.DueDate != null ? closeDate = new Date(i.DueDate) : 'None';
                    var closeDate = i.Availability != null ? closeDate = new Date(i.Availability.EndDate) : 'None';
                    table.push([i.Id, i.Name, dueDate.toLocaleString(), closeDate.toLocaleString(), i.TotalUsersWithSubmissions]);
                });

                vorpal.log(table.toString());
            }
        } else if (args.options['grades']) {

            course = await findCourse(args.course);
            grade = await getCourseGrade(course.OrgUnit.Id);
            var table = new Table({
                head: ['ID', 'Name', 'Grade'],
                chars: { 'top': '-', 'top-mid': '+', 'top-left': '+', 'top-right': '+',
                    'bottom': '-', 'bottom-mid': '+', 'bottom-left': '+', 'bottom-right': '+',
                    'left': '|', 'left-mid': '+', 'mid': '-', 'mid-mid': '+',
                    'right': '|', 'right-mid': '+', 'middle': '|' }
            });

            var gradeDetail;

            grade.map(async function (i) {
                gradeDetail = await getCourseGradeDetail(course.OrgUnit.Id, i.Id);

                table.push([i.Id, i.Name, gradeDetail.DisplayedGrade]);
            });

            vorpal.log(table.toString());
        } else {
            course = await findCourse(args.course);
            var table = new Table({
                chars: { 'top': '-', 'top-mid': '+', 'top-left': '+', 'top-right': '+',
                    'bottom': '-', 'bottom-mid': '+', 'bottom-left': '+', 'bottom-right': '+',
                    'left': '|', 'left-mid': '+', 'mid': '-', 'mid-mid': '+',
                    'right': '|', 'right-mid': '+', 'middle': '|' }
            });
            table.push({ 'Name': course.OrgUnit.Name });
            vorpal.log(table.toString());
        }
        callback();
    });

    vorpal.command('courses', 'list courses as a table').action(async function (args, callback) {
        courses = await courseListing();
        var table = new Table({ head: ["Name", "Code"],
            chars: { 'top': '-', 'top-mid': '+', 'top-left': '+', 'top-right': '+',
                'bottom': '-', 'bottom-mid': '+', 'bottom-left': '+', 'bottom-right': '+',
                'left': '|', 'left-mid': '+', 'mid': '-', 'mid-mid': '+',
                'right': '|', 'right-mid': '+', 'middle': '|' }
        });

        courses.map(function (i) {
            var results = i.OrgUnit.Name.split(' ');
            table.push([results.shift(), results.join(' ')]);
        });

        vorpal.log(table.toString());
        callback();
    });
};

(async function () {

    vorpal.command('login', 'login to slate').action(async function (args, cb) {
        var self = this;

        if (fs.existsSync(path.join(os.homedir(), '.slaterc'))) {
            var configBuffer = fs.readFileSync(path.join(os.homedir(), '.slaterc'));
            config = JSON.parse(configBuffer.toString());
            await initVorpalSlate(config.username, config.password);
        } else {
            var result;
            await this.prompt([{
                type: 'input',
                name: 'username',
                message: 'Username: '
            }, {
                type: 'password',
                name: 'password',
                message: 'Password: '
            }], function (answers) {
                result = answers;
            });

            await initVorpalSlate(result.username, result.password);
        }
    });

    vorpal.delimiter('slate$').use(repl).show().parse(process.argv);
})();

